if (typeof CUI == 'undefined') {
    CUI = {Globals: {}, Dialog : {}};
}

CUI.Globals =   {
    Revision: '',
    ActiveView: '',
    backupStatsRefreshTimer: null,
    shareStatsRefreshTimer: null,
    Filters: [
        {'t' : 'pdf', 'e' : '*.pdf'},
        {'t' : 'office', 'e' : '*.maf;*.oth;*.pptm;*.ott;*.ots;*.otp;*.pptx;*.xltm;*.mar;*.maq;*.oti;*.otf;*.otg;*.otc;*.ppt;*.accdr;*.accdt;*.xlsm;*.xls;' +
             '*.txt;*.xts;*.xlsx;*.accde;*.mam;*.odm;*.odi;*.odf;*.odg;*.odb;*.odc;*.adp;*.odt;*.ods;*.odp;*.doc;*.docm;*.mdt;*.mdw;*.adn;*.accdb;*.docx;*.dotx;' +
             '*.mat;*.mde;*.mdf;*.mda;*.mdb;*.ppl;*.dot;*.dotx;*.dotm;*.xlt;*.xltx;*.pot;*.potx;*.potm;*.one;*.xsn;*.mpt;*.vst;*.pub;' +
              '*.vsd;*.vss;*.vsw;*.vsdx;*.vsdm;*.vssx;*.vssm;*.vstx;*.vstm;*.vsl;*.vdx;*.vtx;*.vsx'},
        {'t' : 'html', 'e' : '*.html;*.htm;*.mht;*.mhtml;*.shtml'},
        {'t' : 'image', 'e' : '*.gif;*.jpg;*.bmp;*.png;*.tif;*.jpe;*.raw;*.pic;*.pct;*.pxr;*.sct;*.ico;*.PSD'},
        {'t' : 'video', 'e' : '*.avi;*.mpg;*.mpeg;*.divx;*.rmvb;*.wmv;*.mov;*.3gp;*.mp4;' +
              '*.rm;*.swf;*.flv;*.mkv;*.3gp'},
        {'t' : 'audio', 'e' : '*.ogg;*.mp3;*.wav;*.flac;*.mpc;*.au;*.aiff;*.wma;' +
              '*.aac;*.m4a;*.ra;*.ape'},
        {'t' : 'email', 'e' :
        '*.pst;*.dbx;*.dbw;*.nsf;*.mbx;*.mbox;*.msf;*.eml;*.emlx'},
        {'t' : 'exe', 'e' : '*.exe;*.dll;*.msi;*.com;*.drv;*.sys;*.cpl;*.ocx;*.msp'},
        {'t' : 'all', 'e' :'*;*.*'}
    ]
};

CUI.Utils = {
	executeFunction: function (functionName, context ) {
        var args = Array.prototype.slice.call(arguments, 2);
        var namespaces = functionName.split(".");
        var func = namespaces.pop();
        for (var i = 0; i < namespaces.length; i++) {
            context = context[namespaces[i]];
        }
        return context[func].apply(context, args);
	},

	//function to create instance of popup dialog
    //parameters:
    //selector : selector to be used by jquerytools
    //conf : configuration to override default configuration setting
    modalDialog : function(selector, conf){
        var overlayConf = {
                //absolute: true,
                api:true,
                left:'center',
                top:'center',
                expose: {
                    color: 'transparent',
                    loadSpeed: 0,
                    closeSpeed : 0,
                    zIndex: 1000
                },
                onClose: function() {
                    var divId;
                    try {
                        divId = $(selector).attr('rel') || selector;
                        jQuery(divId).closest('form')[0].reset();
                    } catch(e) {
                    }
                },
                closeOnClick: false,
                closeSpeed : 0,
                speed : 0,
                oneInstance:false,
                confirmation: {
                    suppress: function(){return false;},
                    message: CUI._('Your changes will be lost. Are you sure you want to cancel?')
                }
            };
        jQuery.extend(true,overlayConf, conf);
        var overlayObject = jQuery(selector).overlay(overlayConf);
        if(overlayObject){
            overlayObject.show = overlayObject.load;
            overlayObject.hide = overlayObject.close;
        }
        return overlayObject;
    },   

    waitDialog : function() {
        var self = this;
        var dlg = self.modalDialog('#loading-div');
        dlg.setHeader = function(waitMsg){
            jQuery('#loading-div .loading-text').html(waitMsg);
        };
        dlg.setFooter = function(waitMsg){
            jQuery('#loading-div .loading-text-small').html(waitMsg);
        };
        return dlg;
    }, 

    alertDialog : function(conf) {
        var self = this;
        var dlg = self.modalDialog('#alert-div',{
            expose: {
                zIndex: 999999
            }
        });
        dlg.show();
        var footerContainer = $('#alert-div .popup-footer-buttons');
        var defaults = {
            buttons: {
                'Ok': function(){}
            }
        };
        $.extend(defaults, conf || {});
        if(typeof defaults.alertIconSrc !== 'undefined') {
            $('#alert-div .alert-icon').attr('src', defaults.alertIconSrc).show();
            $('#alert-div .message').addClass('icon');
        }     
        $('#alert-div .message').html(defaults.message);
        footerContainer.empty();
        var handleClick = function(btn) {
            return function() {
                dlg.hide();
                $('#alert-div .alert-icon').hide();
                //delay added to make sure the alert box is hidden
                //and only then function is called since the client 
                //is syncronous and RPCs could take longer
                setTimeout(function(){defaults.buttons[btn]();},200);
            };
        };
        for(var button in defaults.buttons){
            footerContainer.append('<a class="action-btn secondary-btn padded" style="margin-left:5px;" href="javascript:;" id="' + button + '">' + CUI._(button) + '</a>');
            $('#' + button).click(handleClick(button));
        }            
        return dlg;
    }, 

    browse_dlg : function(conf) {
        var dlg = this.modalDialog('#cui_util_browse_dlg', conf);
        $('#cui_util_browse_cancel').click(function() {
            CUI.Dialog.Browse.close();
        });
        return dlg;
    },

    browse : function(conf){
        $('#cui_util_browse_dlg .step-heading .heading').html(conf.title);
        var oArg = {'div': conf.container,
                    'txt': 'cui_util_browse_path',
                    'showfiles': conf.showFiles,
                    'handlermethod': conf.callback,
                    'data': conf.data,
                    'opts': conf.opts,
                    'changedisplaytext': conf.changedisplaytext};
        var genericFolderTree = new inSyncStaticTree(oArg);
        return genericFolderTree;
    },

    attachTooltip: function(config) {
        $('.tooltip-s-gravity').tipsy({
            gravity: 's',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-n-gravity').tipsy({
            gravity: 'n',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-ne-gravity').tipsy({
            gravity: 'ne',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-se-gravity').tipsy({
            gravity: 'se',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-nw-gravity').tipsy({
            gravity: 'nw',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-sw-gravity').tipsy({
            gravity: 'sw',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-w-gravity').tipsy({
            gravity: 'w',
            live: true,
            opacity: 1,
            html: true
        });

        $('.tooltip-e-gravity').tipsy({
            gravity: 'e',
            live: true,
            opacity: 1,
            html: true
        });
    },

    isEllipsisActive: function(el) {
       return (el.offsetWidth < el.scrollWidth);
    },

    attachTooltipForEllipsis: function($container){
       $('.ellipsis', $container).each(function(){
           if(CUI.Utils.isEllipsisActive(this)){
               $(this).attr('title', CUI.Utils.htmlEncode($(this).text().trim()));
           }
       });
       CUI.Utils.attachTooltip();
    },

    attachTooltipForEllipsisHtml: function($container){
       $('.ellipsis_html', $container).each(function(){
           if(CUI.Utils.isEllipsisActive(this)){
               $(this).attr('title', $(this).html().trim());
           }
       });
       CUI.Utils.attachTooltip();
    },

    initializeScrollBar: function(target) {
        $(window).unbind('resize.scrollbar' + target);
        $(target).jScrollPane(
            {
                horizontalGutter:5,
                verticalGutter:5,
                'showArrows': false
            }
        );
        var api = $(target).data('jsp');          
        var throttleTimeout;
        $(window).bind(
            'resize.scrollbar' + target,
            function()
            {
                if ($.browser.msie) {
                    // IE fires multiple resize events while you are dragging the browser window which
                    // causes it to crash if you try to update the scrollpane on every one. So we need
                    // to throttle it to fire a maximum of once every 50 milliseconds...
                    if (!throttleTimeout) {
                        throttleTimeout = setTimeout(
                            function()
                            {
                                api.reinitialise();
                                throttleTimeout = null;
                            },
                            50
                        );
                    }
                } else {
                    api.reinitialise();
                }
            }
        );
    },

    // Escape special characters for jQuery selectors
    escapeSelector : function(str) {
        if (typeof (str) == 'undefined') {
            return '';
        }
        str = str.replace(/#/g, '\\#')
                 .replace(/;/g, '\\;')
                 .replace(/&/g, '\\&')
                 .replace(/,/g, '\\,')
                 .replace(/\./g, '\\.')
                 .replace(/\+/g, '\\+')
                 .replace(/~/g, '\\~')
                 .replace(/'/g, '\\\'')
                 .replace(/:/g, '\\:')
                 .replace(/\^/g, '\\^')
                 .replace(/\$/g, '\\$')
                 .replace(/\[/g, '\\[')
                 .replace(/\]/g, '\\]')
                 .replace(/\(/g, '\\(')
                 .replace(/\)/g, '\\)')
                 .replace(/\=/g, '\\=')
                 .replace(/\//g, '\\/')
                 .replace(/\{/g, '\\{')
                 .replace(/\}/g, '\\}')
                 .replace(/ /g, '\\ ');
        return str;
    },
    
    htmlEntityEncode : function(s){
        return s.replace(/'/g, '&apos;').replace(/"/g, '&quot;');
    },

    htmlEncode: function (value) {
        //create a in-memory div, set it's inner text(which jQuery automatically encodes)
        //then grab the encoded contents back out.  The div never exists on the page.
        return $('<div/>').text(value).html();
    },

    formatString: function() {
        var s = arguments[0];
        for (var i = 0; i < arguments.length - 1; i++) {
            var reg = new RegExp("\\{" + i + "\\}", "gm");
            s = s.replace(reg, arguments[i + 1]);
        }
        return s;
    }

};

CUI.RPC = {
	callmethod: function(method, args){
		try {
			var rc = method.split('.');
			var cls = rc[0],
			    fn = rc[1],
			    target;
			if (cls === 'cuiserver') { 
                target = cuiserver; 
            } else if (cls === 'summary') { 
                target = cui_summary; 
            } else if (cls === 'share') {
                target = cui_share;
            } else if (cls == 'preferences') { 
                target = cui_preferences; 
            } else if (cls == 'myaccount') { 
                target = cui_myaccount; 
            } else if (cls == 'useraction') {
                target = cui_useraction;
            }
            var response = {};
            if (typeof args == 'undefined') {
                response = target[fn]();
            } else {
                response = target[fn](args);
            }
			return response;
		} catch (e) {
			return {'status' : -1, 'msg' : e.message };
		}
	}
};

var inSyncStack = function() {
    this.init();
};

inSyncStack.prototype = {
    init : function(){
        var self = this;
        self.__stack = [];        
    },
    push: function(element){
        var self = this;
        self.__stack.push(element);
    },
    pop: function(){
        var self = this;
        return self.__stack.pop();
    },
    isempty: function(){
        var self = this;        
        if (self.__stack.length == 0) {
            return true;
        }
        return false; 
    }
};
