CUI.Preferences = {

	init : function(){
        CUI.Globals.ActiveView = 'preferences';
        var self = this;
        self._bindEvents();
        self.initiateSliders();
    },

    _bindEvents: function(){
        var self = this;
        $(".backup-resource-preferences").change(function(e){
            self.updateBackupPreferences(e);
            var selectedPriority = $(this).find('option:selected');
            var color = selectedPriority.css('color'),
            note = selectedPriority.attr('note');
            $(this).css({'color' : color});
            $('.cpupriority-note').html(note);
        });

        $('#data-privacy-settings').change(function() {
            self.updateDataPrivacySettings();
        });
        $('#proxy-settings').on('click', '#btn-configure-proxy', function(e){
            e.preventDefault();
            var res = CUI.RPC.callmethod('preferences.launchProxyWizard');
            self.reloadProxyConfig();
        });
        // Block selected network
        $('#proxy-settings').on('click', '#btn-block-network', function(e){
            e.preventDefault();
            if (!$(this).hasClass('disabled')) {
                var selectedRows = $('#recent-networks-table tbody tr.selected');
                var data = {network: selectedRows.eq(0).data(), action: 'BLOCK'};
                var res = CUI.RPC.callmethod('preferences.updateNetworkBlocking', data);
                self.reloadProxyConfig();
            }
        });
        // Unblock selected network
        $('#proxy-settings').on('click', '#btn-unblock-network', function(e){
            e.preventDefault();
            if (!$(this).hasClass('disabled')) {
                var selectedRows = $('#blocked-networks-table tbody tr.selected');
                var data = {network: selectedRows.eq(0).data(), action: 'UNBLOCK'};
                var res = CUI.RPC.callmethod('preferences.updateNetworkBlocking', data);
                self.reloadProxyConfig();
            }
        });
        // Highlight network and enable block/unblock button
        $('#proxy-settings').on('click', '.table-container tbody tr', function(e){
            e.preventDefault();
            if (!$(this).hasClass('admin-blocked')) {
                $(this).closest('tbody').find('tr.selected').removeClass('selected');
                $(this).addClass('selected');
                $(this).closest('.field-group').find('.action-btn').removeClass('disabled');
            }
        });
        // Initially enable block/unblock button if any network is highlighted
        $('#proxy-settings .table-container').each(function(i){
            if ($('tr.selected', this).length) {
                $(this).closest('.field-group').find('.action-btn').removeClass('disabled');
            }
        });

        self.updateLastValue();

        CUI.Dialog.Browse = CUI.Utils.browse_dlg({
            expose: {zIndex: 10002},
            onClose: function(e){
                $('#cui_util_browse_output').val('');
                // Destroy the trees
                if($.tree.reference('#cui_util_browse_tree')){
                    $.tree.reference('#cui_util_browse_tree').destroy();
                    $('#cui_util_browse_tree').empty();
                }
            },
            confirmation : {
                suppress: function(){
                    return true;
                }
            }
        });
        $('.field-tree').hide();
        $('#cui_util_browse_tree').hide();
        $('#cui_util_browse_dlg .popup-footer a').hide();
        $('#cui_util_browse_cancel').hide();
        $('#cui_util_browse_next').hide();
        $('#cui_util_browse_form').hide();
        $('#cui_util_browse_dlg').hide();

        $('#cui_util_browse_save').click(function(e){
            e.preventDefault();
            var rootsharejson = $.parseJSON($('#cui_util_browse_output').val());
            if(!rootsharejson) {
                CUI.Utils.alertDialog({
                    message: CUI._('Please select new inSync share folder location.')
                });
                return false;
            } else if (rootsharejson.attributes.rel == 'file'){
                CUI.Utils.alertDialog({
                    message: CUI._('Please select new inSync share folder location.')
                });
                return false;
            } else {
                var msgCofirm, msgAlert;
                msgAlert = CUI._('Successfully updated inSync Share location.');
                delete rootsharejson['disabled'];
                rootsharejson.checked = true;
                rootsharejson.state = 'open';
                rootsharejson.attributes.rel = 'folder';
                rootsharejson.attributes.id = rootsharejson.attributes.id + "1";
                var path = rootsharejson.attributes.path;
                var data = {};
                data['path'] = path;
                $('#cui_util_browse_path_div').removeClass('hide');
                var response = CUI.RPC.callmethod('preferences.moveShareLocation', data);
                if(response['status'] == -1){
                    CUI.Utils.alertDialog({message: response['msg']});
                } else{
                    CUI.Dialog.Browse.close();
                    $('#user-sharehome').html(response['ush']);
                    self.showWaitDialog();
                }
            }
        });
    },

    initiateSliders: function() {
        $('.intv-text').html($('#syncInterval option:selected').text());
        $('.bw-avail').html($('select#maxBandwidth option:selected').text());
        $('.cpu-priority-text').html($('select#cpuPriority option:selected').text());

        if($('#syncInterval').length) {
            $('#syncInterval').selectToUISlider({
                labels: 5,
                labelSrc: 'text',
                tooltip: false,
                sliderOptions: {
                    stop: function(e, ui) {
                        $('#syncInterval').trigger('change');
                    }
                }
            });
        }

        if($('select#maxBandwidth').length) {
            $('select#maxBandwidth').selectToUISlider({
                labels: 5,
                labelSrc: 'text',
                tooltip: false,
                sliderOptions : {
                    stop : function(e, ui) {
                        $('select#maxBandwidth').trigger('change');
                    }
                }
            });
        }

        if($('select#cpuPriority').length) {
            $('select#cpuPriority').selectToUISlider({
                labels: 3,
                labelSrc: 'text',
                tooltip: false,
                sliderOptions : {
                    stop : function(e, ui) {
                        $('select#cpuPriority').trigger('change');
                    }
                }
            });
        }
    },

	updateLastValue: function(){
        $('.backup-resource-preferences').each(function(idx, val){
            $(val).data('lastValue', $(val).val());
        });
	},

    postPreferenceUpdate: function(esource, response){
        var self = this;
        if(typeof response['msg'] !== 'undefined') {
            $('.backup-pref-notify').html(response['msg']).show();
            setTimeout(function(){
                $('.backup-pref-notify').fadeOut();
            }, 1000);
        }
        if(response['status'] == -1){
            $(esource).val($(esource).data('lastValue'));
            $(esource).closest('.controls').find('.pref-slider-text').html($(esource).find('option:selected').text());
        } else {
            self.updateLastValue();
        }
    },

    updatePauseInterval:function(e){
        var data = {}, self=this;
        var source = e.currentTarget;
        var interval = $("#pauseDuration").val();
        data['pause_interval'] = interval;
        var response = CUI.RPC.callmethod('summary.update_pause_interval', data);
        self.postPreferenceUpdate(source, response);
    },

    updateBackupPreferences: function(e){
        var rc, data = {}, source=e.currentTarget, key, oldvalue, self=this;
        var id = $(source).attr('id');
        oldvalue = $(source).data('lastValue');
        if (id == 'pauseDuration'){
            // Need to call tray APIs on top of updating inSyncConfig. 
            // new Qt slot update_pause_interval should get called instead
            // of updateBackupPreferences.
            self.updatePauseInterval(e);
            return;
        } else if($(source).closest('.battery-content').length) {
            if (id == 'fullScrPause') {
                key = 'OPTIMAL_MODE';
                data[key] = $('input.pause-full-screen').is(':checked') ? '1' : '0';
            } else {
                key = 'MIN_BATTERY';
                if (!$('input.pause-option').is(':checked')) {
                    data[key] = '0';
                    $('#btrylvl').attr('disabled', true);
                } else {
                    data[key] = $('#btrylvl').val();
                    $('#btrylvl').attr('disabled', false);
                }
            }
        } else {
            if(id == 'syncInterval') {
                key = 'INTERVAL_IDX';
            } else if(id == 'maxBandwidth') {
                key = 'BANDWIDTH';
            } else {
                key = 'PRIORITY';
            }
            data[key] = $(source).val();
        }
        var response = CUI.RPC.callmethod('preferences.updateBackupPreferences', data);
        self.postPreferenceUpdate(source, response);
    },

    _updatePrivacyCheckbox: function(disableSrvRestore){
        if (disableSrvRestore) {
            $("#privacy-select").removeAttr('checked');
        } else {
            $("#privacy-select").attr('checked', 'checked');
        }
    },
    updateDataPrivacySettings: function(){
        var msgCofirm, msgAlert,  self = this,
        data = {},
        disableSrvRestore = ! $("#privacy-select").is(':checked');
        self._updatePrivacyCheckbox(!disableSrvRestore);
        data['disable'] = disableSrvRestore;
        if(disableSrvRestore){
            msgCofirm = CUI._('Do you want to disallow administrators to view or download your data?');
            msgAlert = CUI._('Successfully updated privacy settings - administrators can not view or download your data.');
        } else{
            msgCofirm = CUI._('Do you want to allow administrators to view or download your data?');
            msgAlert = CUI._('Successfully updated privacy settings - administrators can view or download your data.');
        }
        CUI.Utils.alertDialog({
            message: msgCofirm,
            buttons: {
                'No': function() {},
                'Yes': function(){
                    var response = CUI.RPC.callmethod('preferences.disableSrvRestore', data);
                    self._updatePrivacyCheckbox(response['disabled']);
                    if(response['status'] == -1){
                        CUI.Utils.alertDialog({message: response['msg']});
                    } else {
                        CUI.Utils.alertDialog({message: msgAlert});
                    }
                }
            }
        });
    },

    reloadProxyConfig: function(){
        var response = CUI.RPC.callmethod('preferences.reloadProxyConfig');
        if (response['status'] != -1 ){
            $('#proxy-settings').html(response['html']);
        }
    },

    showFileSystemFolders: function(){
        $('.field-tree').show();
        $('#cui_util_browse_tree').show();
        $('#cui_util_browse_dlg .popup-footer a').hide();
        $('#cui_util_browse_cancel').show();
        $('#cui_util_browse_next').hide();
        $('#cui_util_browse_save').show();
        $('#cui_util_browse_dlg .popup-content > .field-path').addClass('hide');
        $('#cui_util_browse_dlg .popup-content > .field-tree').addClass('classic');

        $('#cui_util_browse_dlg .step-heading .heading').html(CUI._('Select Share Folder Location'));
    },

    showWaitDialog: function(){
        var self = this;
        var wait = CUI.Utils.waitDialog();
        var headermsg = CUI._('Please wait while we update your inSync Share folder location. This will take a while...'); 
        wait.setHeader(headermsg);
        var footermsg = CUI._('Copying data...');
        wait.setFooter(footermsg);
        wait.show();

        var copyInterval = setInterval(function(){
            var response = CUI.RPC.callmethod('preferences.getShareFolderStatus');
            if(response['status'] == 1){
                clearInterval(copyInterval);
                $('#user-sharehome').html(response['ush']);
                wait.hide();
                CUI.Utils.alertDialog({message: response['msg']});
            }
        }, 5000);
    },

    showGeneralTab: function() {
        $("#pref-tab li").find('a[href=#backup-pref]').tab('show');
    },
    showNetworkTab: function() {
        $("#pref-tab li").find('a[href=#proxy-settings]').tab('show');
    }
};
