/*
- Details dialogues
- Backup progress, details, pause, cancel, resume
- Troubleshoot
- Logs
*/

CUI.Summary =  {
    init : function(){
        CUI.Globals.ActiveView = 'summary';
        var self = this;
        self.ui_folderids = null;
        self.accumulated_fstats = {};
        self.excluded_shares = [];
        self.bindEvents();
        CUI.Summary.Logs.init();
        self.refreshBackupShareState();
    },

    bindEvents: function() {        
        var self = this;
        var pauseHoverTime;
        self.isDetailsViewOpened = false;
        self._adjustHeightCallback = self.adjustBackupContentHeights('.backup-info-container', '.backup-bottom-content');

        $("#backup-summary-stats").on('click', '#lastbackup-details', function(e){
            e.preventDefault();
            self.viewBackupDetails();
        });  

        self.viewBackupDetails(false);
        setTimeout(self._adjustHeightCallback, 150);

        $('.backup-progress-bar').on('click', '.action-button', function(e){
            e.preventDefault();
            var method = $(this).attr('action-name');
            CUI.RPC.callmethod(method);
            self.refreshBackupShareState();
        });

        $('.backup-progress-bar').on('click', '.confirm-action-button', function(e){
            e.preventDefault();
            var data = {};

            // Do not take any action if button is inactive
            if ($(this).children('#client-syncnow').hasClass('action-btn-inactive')) {
               return;
            }

            data['force'] = false;
            var method = $(this).attr('action-name');
            var iconSrc = $(this).attr('icon-src');
            var jsActionName = $(this).attr('js-action-name');
            var response = CUI.RPC.callmethod(method,data);
            if (response['status'] === -1 ){
                CUI.Utils.alertDialog({message: response['msg'],alertIconSrc : iconSrc});
            }
            else if (response['status'] === 1 ){
                CUI.Utils.alertDialog({
                                    message: response['msg'],
                                    alertIconSrc : iconSrc,
                                    buttons : {
                                        'No': function() {},
                                        'Yes': function(){
                                            var data = {};
                                            data['force'] = true;
                                            var confirmedResponse = CUI.RPC.callmethod(method,data);
                                            if (confirmedResponse['status'] === 0){
                                                if (typeof jsActionName !== 'undefined'){
                                                    CUI.Utils.executeFunction(jsActionName,self);
                                                }
                                                self.refreshBackupShareState();
                                            }
                                        }
                                    }
                                });
            }
            else if (response['status'] === 0){
                if (typeof jsActionName !== 'undefined'){
                    CUI.Utils.executeFunction(jsActionName,self);
                }
                self.refreshBackupShareState();
            }
        });


        $('#backup-summary-stats').on('click', 'a.backup-action-stop', function(e){
            e.preventDefault();
            e.stopPropagation();
            CUI.Utils.alertDialog({
                message: CUI._('Are you sure you want to stop the backup?'),
                buttons: {
                    'No': function() {},
                    'Yes': function(){
                        var response = CUI.RPC.callmethod('summary.stopOngoingBackup');
                        self.refreshBackupShareState();
                    }
                }
            });
        });

        $('.table-container-parent-div').on('click', '.folder-name a.edit', function(e){
            e.preventDefault();
            var selected_row = $(e.currentTarget).parents('td').parents('tr');
            var backup_folderid =  selected_row.attr('data-share-id');
            var email_client = selected_row.attr('data-email-client');
            var isMandatory = $(this).hasClass('mandatory');
            $(this).data({
                client_params: {
                    action: 'show-tree-view',
                    backup_folderid: backup_folderid,
                    email_client: email_client,
                    is_mandatory: isMandatory
                }
            });
        });

        self.backupDetails = CUI.Utils.modalDialog('#backup-details-window');
        
        //set the timer event only once.
        if(CUI.Globals.backupStatsRefreshTimer == null){ 
            self.setRefreshTimeout();
            CUI.Globals.backupStatsRefreshTimer = true;    
        }

        $('#client-syncnow').click(function() {
            var $detailsView = $('.details-view');
            $detailsView.hide();
            self._adjustHeightCallback();
        });
    },

    /***
     * Update only data without recreating the DOM.
     */
    updateBackupStats: function(stats) {
        var self = this;
        $('#nextBackupDate').text(stats.nextBackupDate);
    },

    /**
     * Replace the DOM elements with new backup details content.
     */
    viewBackupDetails: function(state){
        var self = this;
        var response = CUI.RPC.callmethod('summary.getBackupDetails');
        if (response['status'] != -1 ){
            $('.details-view').html(response['html']);
            // force specified state or toggle
            if (arguments.length) { 
                state = !!state;
                $('.details-view .expanded-view').stop(true).toggle(state).promise().done(self._adjustHeightCallback);
                self.isDetailsViewOpened = state;
            } else if(self.isDetailsViewOpened) {
                $('.details-view .expanded-view').show().stop(true).slideUp({step: self._adjustHeightCallback});
                self.isDetailsViewOpened = false;
            } else {
                $('.details-view .expanded-view').stop(true).slideDown({step: self._adjustHeightCallback});
                self.isDetailsViewOpened = true;
            }
            CUI.Utils.attachTooltip();
        }
    },

    adjustBackupContentHeights: function(topContent, bottomContent) {
        var $topContent = $(topContent);
        var $bottomContent = $(bottomContent);
        var overflow;
        return function() {
            $bottomContent.css('top', $topContent.position().top + $topContent.outerHeight(true) + 12);
            var $backupTable = $('#backup-shares-list', $bottomContent);
            var $backupTableContainer = $backupTable && $backupTable.parent();
            var overflow = $backupTableContainer && $backupTable.outerHeight(true) > $backupTableContainer.height();
            $('#backup-accshares', $bottomContent).toggleClass('overflow', overflow);
        };
    },

    refreshBackupShareState: function(keepAlive) {
        var self = this, res, inc_stats, data, response, stats;
        // Do not refresh stats if summary is not the active view.
        if (CUI.Globals.ActiveView == 'summary') {
            data = {'inc_stats': true}; 
            response = CUI.RPC.callmethod("summary.getSyncStats", data);

            if (response['status'] == 0) {
                stats = $.parseJSON(response['stats']);
                stats.isDetailsViewOpened = self.isDetailsViewOpened;

                // Render stats
                $('#summary-progress-pre').html($('#summary-progress-pre-tmpl').tmpl(stats));
                $('#summary-progress-post').html($('#summary-progress-post-tmpl').tmpl(stats));
                
                if(!stats.isBackupActive) {
                    // Hide progress bar
                    $('#summary-progress-container').hide();

                    // Show details view
                    $('.details-view').show();

                    // Enable the Backup button
                    if ($('#client-syncnow').hasClass('action-btn-inactive')) {
                        $('#client-syncnow').removeClass('action-btn-inactive');
                        self.viewBackupDetails(self.isDetailsViewOpened);
                    } else {
                        self.updateBackupStats(stats);
                    }

                } else {
                    // Disable the Backup button
                    $('#client-syncnow').toggleClass('action-btn-inactive', !stats.backupPaused);

                    // Hide details
                    $('.details-view').hide();

                    // Render progress bar
                    if (stats.backupState == 'Synchronizing' && !stats.isBackupPaused && stats.estimateDone && !stats.estimatedFilesExceeded) {
                        $('#summary-progress-container .striped').hide();
                        $('#summary-progress-container .progress-bar').css('width', stats.percent+'%').show();
                    } else if (!stats.backupPaused) {
                        // Show stripped progress bar when backup is in Initializing or Finalizing phase
                        $('#summary-progress-container .progress-bar').hide();
                        $('#summary-progress-container .striped').show();
                    } else {
                        // Show empty progress bar when backup is paused
                        $('#summary-progress-container .striped').hide();
                        $('#summary-progress-container .progress-bar').hide();
                    }

                    // Show action buttons
                    if (stats.pauseAllowed){
                        if (stats.backupPaused) {
                            $('.backup-action-resume').show()
                            $('.backup-action-pause').hide();
                            $('.backup-action-stop').hide();
                        } else {
                            $('.backup-action-resume').hide();
                            $('.backup-action-pause').show();
                            $('.backup-action-stop').show();
                        }                
                    } else {
                        $('.backup-action-resume').hide();
                        $('.backup-action-pause').hide();
                        $('.backup-action-stop').show();
                    }
                    // Finally, show the container
                    $('#summary-progress-container').show();
                }

                // Post processing
                $('.backup-status-heading').toggleClass('inactive', !stats.isBackupActive);

            }
            self.reloadBackupFolders();
            self._adjustHeightCallback();
        }
        if (keepAlive) {
            self.setRefreshTimeout();
        }
    },

    setRefreshTimeout: function(){
        var self = this;
        setTimeout(function(){
            self.refreshBackupShareState(true);
        }, 1000);
    },

    
    updateFolderIdsCache: function(){
        var self = this;
        var shareids = [];
        var allrows = $('#backup-shares-list tbody').find('tr');
        $(allrows).each(function(idx, val){
            shareids.push($(val).attr('data-share-id'));
        });
        self.ui_folderids = JSON.stringify(shareids);
    },

    reloadBackupFolders: function(){
        var self = this;
        if(!self.ui_folderids) {
            self.updateFolderIdsCache();
        }
        var data = {};
        data['shareids'] = self.ui_folderids;
        data['accumulated_fstats'] = self.accumulated_fstats;
        data['excluded_shares'] = self.excluded_shares;
        var response = CUI.RPC.callmethod('summary.reloadBackupFolders', data);
        var $container = $('.table-container-parent-div');
        if(response['status'] == 0 && response['reload'] == true){
            self.accumulated_fstats = response['accumulated_fstats'];
            self.excluded_shares = response['excluded_shares'];
            $container.html('');
            $.fn.tipsy.revalidate();
            $container.html(response['html']);
            self.updateFolderIdsCache();
            CUI.Utils.attachTooltipForEllipsis($container);
        }
    },
};

CUI.Summary.Logs = {
    init : function(){
        CUI.Globals.ActiveView = 'summary';
        var self = this;
        self.logscounter = 0;
        self.logfiles = {};
        self.logindex = 0;
        self.bindEvents();
    },

    bindEvents: function() {
        var self = this;
        self.LogsViewer = CUI.Utils.modalDialog('#backup-logs-window');
        $("div.details-view").on('click', '.view-logs-btn .view-logs', function(e){
            e.preventDefault();
            self.openLogViewer();
        });
        $(".btn-client-logs-prev").click(function(e){
            e.preventDefault();
            if($(this).hasClass('disabled')){
                return false;
            }
            self.logindex = self.logindex > 0? self.logindex - 1: 0;
            self.loadLogFile();
        });
        $(".btn-client-logs-next").click(function(e){
            e.preventDefault();
            if($(this).hasClass('disabled')){
                return false;
            }
            self.logindex = self.logindex + 1 >= self.logscount ? self.logscount - 1: self.logindex + 1;
            self.loadLogFile();

        });

    },

    openLogViewer: function(){
        var self = this;
        var response = CUI.RPC.callmethod('summary.listLogFiles');
        if (response['status'] != -1 ){
            var logscount = response['count'];
            if (logscount == 0) {
                return;
            } else {
                self.logscount = logscount;
            }
            self.logfiles = $.parseJSON(response['files']);
            self.logindex = self.logscount -1;
            self.loadLogFile();
            self.LogsViewer.show();
        }
    },

    loadLogFile: function(){
        var self = this;
        var data = {};
        data['logfile'] = self.logfiles[self.logindex];
        var response = CUI.RPC.callmethod('summary.readLogFile', data);
        if (response['status'] != -1){
            $('#log-file-index-label').html(CUI._("Log File")+ " " + (self.logindex+1) + " " + CUI._("of") + " " + self.logscount);
            $('#backup-logs-content').val(response['html']);
            $('input.log-file-path').val(data['logfile']);
        }
        if (self.logindex+1 === self.logscount) {
            $(".btn-client-logs-next").addClass('disabled');
        } else {
            $(".btn-client-logs-next").removeClass('disabled');
        }
        if (self.logindex+1 === 1) {
            $(".btn-client-logs-prev").addClass('disabled');
        } else {
            $(".btn-client-logs-prev").removeClass('disabled');
        }
    },
};
