var inSyncTree = function(oArg) {
    this.instance = jQuery.tree.create();
    this.init(oArg);
};

inSyncTree.prototype = {
    init: function (oArg) {
        var treeDiv = oArg.div,
            outTextbox = oArg.txt,
            dispTextbox = oArg.txt2,
            hasFiles = oArg.files,
            fileCallback = oArg.cbfn,
            path = oArg.path,
            opts = oArg.opts,
            show_path = '',
            baseUrl = oArg.url,
            url;

        if (typeof(hasFiles) == 'undefined') {
            hasFiles = false;
        }
        if (typeof(fileCallback) == 'undefined') {
            fileCallback = false;
        }
        if (typeof(path) != 'undefined') {
            url = baseUrl + '&path=' + encodeURIComponent(path);
        } else {
            url = baseUrl;
        }

        var selector = '#' + treeDiv;
        var defaultOpts = {
            ui : {
                    theme_path : 'css/jsTree.classic.css',
                    theme_name : 'classic',
                    animation : 2
            },
            data : {
                type : 'json',
                async : true,
                opts : {
                    method : 'POST',
                    url : url
                }
            },
            types : {
                "root" : {
                    icon : {
                        image : "img/drive.png"
                    }
                },
                "folder" : {
                    icon : {
                        image : "img/folder1.png"
                    }
                },
                "file" : {
                    icon : {
                        image : "img/file.png"
                    }
                },
                "mbox" : {
                    icon : {
                        image : "img/mbox.png"
                    }
                }
            },
            callback : {
                beforeopen : function(n, t) {
                    var path = $(n).attr('id');
                    //path = encodeURIComponent(path);
                    inSyncTree.DisplayText(n, t, dispTextbox, outTextbox);
                    t.settings.data.opts.url = baseUrl + '&path=' + encodeURIComponent(path);
                    return;
                },
                ondata : function(data, t){
                    // Separate file and folder lists if required
                    if (hasFiles) {
                        var folders = data[0],
                            folders_files = data[1];
                        if (fileCallback) {
                            fileCallback(folders_files, t);
                            return folders;
                        } else {
                            return folders_files;
                        }
                    } else {
                        return data;
                    }
                },
                onopen : function(n, t) {
                    $('#' + treeDiv + ' a.clicked').removeClass('clicked');
                    var id = $(n).attr('id'),
                        nid = DRP.Utils.escapeSelector(id);
                    $('#' + nid + ' a:first' ).addClass('clicked');
                    $('#' + nid + ' a:first' ).focus();
                    inSyncTree.DisplayText(n, t, dispTextbox, outTextbox);
                },
                onselect : function(n, t) {
                    inSyncTree.DisplayText(n, t, dispTextbox, outTextbox);
                    t.refresh(n);
                }
            }
        };
        var config;
        // Recursively merge default and tree specific options
        if (typeof(opts) != 'undefined') {
            config = jQuery.extend(true, defaultOpts, opts);
        } else {
            config = defaultOpts;
        }
        this.instance.init(selector, config);
    }
};

var inSyncStaticTree = function(oArg) {
    this.instance = jQuery.tree.create();
    this.init(oArg);
};

// Restore GUI specific code marked as !
inSyncStaticTree.prototype = {
    init : function(oArg) {
        var self = this;
        var treeDiv = oArg.div;
        var treeData = oArg.data;
        var outTextbox = oArg.txt;
        var dispTextbox = oArg.txt2;
        var opts = oArg.opts;
        var show_path = oArg.path;
        var showfiles = oArg.showfiles;
        var handlermethod = oArg.handlermethod;
        var changedisplaytext = oArg.changedisplaytext;        

        var selector = '#' + treeDiv;
        var defaultOpts = {
            ui : {
                    theme_path : 'css/jsTree.classic.css',
                    theme_name : 'classic'
                    //animation : 2
            },
            data : {
                type : 'json',
                async : true,
                opts : {
                    'static': treeData
                }
            },
            callback : {
                beforeopen : function(n, t) {
                    var path = $(n).attr('path');
                    var traverse = $(n).attr('traverse');
                    var data = {};
                    data['showfiles'] = showfiles;
                    data['path'] = path;
                    data['traverse'] = traverse;
                    // Send check status of parent to python process and set check 
                    // status of children from python rather than accomplishing the same by DOM processing.
                    if (opts.custom.refresh_onopen == true){
                        var l_tree = $.tree.reference(n);
                        var node = l_tree.get_node(n);
                        var checked = node.children("a").eq(0).hasClass("checked");
                        data['parent_checked'] = checked;                        
                    }
                    if(typeof(opts.custom.check_config_exts) != 'undefined'){
                        data['check_config_exts'] = opts.custom.check_config_exts;
                        data['backup_folderid'] = $('#backup-folder-name').val();
                    }
                    if ($(n).attr('rel') == 'file'){
                        t.settings.data.opts['static'] = [];
                    } else {
                        var response = handlermethod.call(null, data);
                        t.settings.data.opts['static'] = response;                        
                    }
                    return;
                },                
                onselect : function(n, t) {
                    // Selection should be able to set the output text and display text.
                    inSyncTree.DisplayText(n, t, dispTextbox, outTextbox, changedisplaytext);
                },
                onopen : function(n, t) {
                    return; 
                }
            }
        };
        var config;
        // Recursively merge default and tree specific options
        if (typeof(opts) != 'undefined') {
            config = jQuery.extend(defaultOpts, opts);
        } else {
            config = defaultOpts;
        }
        this.instance.init(selector, config);
    }
};

//Show display name in the display textbox
inSyncTree.DisplayText = function(n, t, dispText, outText, changedisplaytext) {
    if (changedisplaytext){
        var dirname = $(n).attr('dname');
        jQuery('#' + outText).val(dirname);                
        var id = $(n).attr('id');
        var nid = CUI.Utils.escapeSelector(id);
        if($('#' + nid).attr('allowselection') === 'false'){ // Do allow the "Quick Configuration" and "My Computer" entries to be selected
            $('#cui_util_browse_output').val('');
            $('#' + nid + ' > a').removeClass('clicked');
        }else{
            $('#cui_util_browse_output').val(JSON.stringify(t.get(n, 'json', {exclude_children: true})));
        }
    }
};
